@@ -73,6 +73,10 @@ def test_fetch():
             "initial": {},
             "passthrough": []
         },
+        "resolver": {
+            "maximum_combinations": 10,
+            "maximum_attempts": 15,
+        },
         "command": {
             "max_content_width": 90,
             "verbosity": "info",
@@ -133,6 +137,10 @@ def test_fetch_with_personal():
             },
             "passthrough": []
         },
+        "resolver": {
+            "maximum_combinations": 10,
+            "maximum_attempts": 15,
+        },
         "command": {
             "max_content_width": 90,
             "verbosity": "info",
