@@ -5,8 +5,10 @@ import base64
 import hashlib
 
 import pytest
+import six
 
 import wiz.definition
+import wiz.package
 import wiz.utility
 from wiz.utility import Requirement
 
@@ -17,6 +19,12 @@ def mocked_extract_version_ranges(mocker):
     return mocker.patch.object(wiz.utility, "extract_version_ranges")
 
 
+@pytest.fixture()
+def mocked_match(mocker):
+    """Return mocked wiz.utility.match function."""
+    return mocker.patch.object(wiz.utility, "match")
+
+
 @pytest.mark.parametrize("element", [
     "This is a string",
     42,
@@ -37,7 +45,7 @@ def mocked_extract_version_ranges(mocker):
 def test_encode_and_decode(element):
     """Encode *element* and immediately decode it."""
     encoded = wiz.utility.encode(element)
-    assert isinstance(encoded, bytes)
+    assert isinstance(encoded, six.string_types)
     assert element == wiz.utility.decode(encoded)
 
 
@@ -239,7 +247,7 @@ def test_extract_version_ranges(requirement, expected):
 ])
 def test_extract_version_ranges_error(requirement, expected):
     """Fail to extract version ranges from requirements."""
-    with pytest.raises(wiz.exception.InvalidRequirement) as error:
+    with pytest.raises(wiz.exception.RequirementError) as error:
         wiz.utility.extract_version_ranges(requirement)
 
     assert expected in str(error)
@@ -292,7 +300,7 @@ def test_update_minimum_version(version, ranges, expected):
 
 def test_update_minimum_version_fail():
     """Fail to update range with minimum value."""
-    with pytest.raises(wiz.exception.InvalidRequirement) as error:
+    with pytest.raises(wiz.exception.RequirementError) as error:
         wiz.utility._update_minimum_version((1, 2, 3), [(None, (1,))])
 
     assert (
@@ -348,7 +356,7 @@ def test_update_maximum_version(version, ranges, expected):
 
 def test_update_maximum_version_fail():
     """Fail to update range with maximum value."""
-    with pytest.raises(wiz.exception.InvalidRequirement) as error:
+    with pytest.raises(wiz.exception.RequirementError) as error:
         wiz.utility._update_maximum_version((1, 2, 3), [((2,), None)])
 
     assert (
@@ -410,7 +418,7 @@ def test_update_version_ranges(excluded, ranges, expected):
 
 def test_update_version_ranges_fail():
     """Fail to update version ranges from excluded version range."""
-    with pytest.raises(wiz.exception.InvalidRequirement) as error:
+    with pytest.raises(wiz.exception.RequirementError) as error:
         wiz.utility._update_version_ranges(((0,), (3,)), [((1,), (2,))])
 
     assert (
@@ -568,3 +576,136 @@ def test_deep_update(mapping1, mapping2, expected):
     assert wiz.utility.deep_update(mapping1, mapping2) == expected
     assert mapping1 == expected
     assert mapping2 == _mapping2
+
+
+@pytest.mark.parametrize("package_data, variant_index, requirement, expected", [
+    ({"identifier": "A"}, None, Requirement("A"), Requirement("::A")),
+    (
+        {"identifier": "A", "version": "2"}, None,
+        Requirement("A > 1"), Requirement("::A > 1")
+    ),
+    (
+        {"identifier": "A", "namespace": "foo"}, None,
+        Requirement("A"), Requirement("foo::A")
+    ),
+    (
+        {"identifier": "A", "namespace": "foo::bar"}, None,
+        Requirement("A"), Requirement("foo::bar::A")
+    ),
+    (
+        {"identifier": "A", "variants": [{"identifier": "V1"}]}, 0,
+        Requirement("A[V1]"), Requirement("::A[V1]")
+    )
+], ids=[
+    "simple",
+    "with-version",
+    "with-namespace",
+    "with-long-namespace",
+    "with-variant",
+])
+def test_sanitize_requirement(
+    mocked_match, package_data, variant_index, requirement, expected
+):
+    """Return qualified requirement depending on package's namespace."""
+    mocked_match.return_value = True
+    package = wiz.package.Package(
+        wiz.definition.Definition(package_data), variant_index=variant_index
+    )
+    assert wiz.utility.sanitize_requirement(requirement, package) == expected
+
+
+def test_sanitize_requirement_error(mocked_match):
+    """Fail to return qualified requirement."""
+    mocked_match.return_value = False
+    package = wiz.package.Package(
+        wiz.definition.Definition({"identifier": "B"})
+    )
+
+    with pytest.raises(ValueError) as error:
+        wiz.utility.sanitize_requirement(Requirement("A"), package)
+
+    assert "Requirement 'A' is incompatible with package 'B'" in str(error)
+
+
+@pytest.mark.parametrize("package_data, variant_index, requirement, expected", [
+    ({"identifier": "A"}, None, Requirement("A"), True),
+    ({"identifier": "B"}, None, Requirement("A"), False),
+    ({"identifier": "A", "version": "1"}, None, Requirement("A>=1"), True),
+    ({"identifier": "A", "version": "0.1.0"}, None, Requirement("A>=1"), False),
+    ({"identifier": "A", "namespace": "ba"}, None, Requirement("A"), True),
+    ({"identifier": "A", "namespace": "ba"}, None, Requirement("ba::A"), True),
+    ({"identifier": "A", "namespace": "to"}, None, Requirement("ba::A"), False),
+    (
+        {"identifier": "A", "variants": [{"identifier": "V1"}]}, 0,
+        Requirement("A"), True
+    ),
+    (
+        {"identifier": "A", "variants": [{"identifier": "V1"}]}, 0,
+        Requirement("A[V1]"), True
+    ),
+    (
+        {"identifier": "A", "variants": [{"identifier": "V1"}]}, 0,
+        Requirement("A[V3]"), False
+    ),
+], ids=[
+    "matching-identifier",
+    "non-matching-identifier",
+    "matching-version",
+    "non-matching-version",
+    "matching-namespace1",
+    "matching-namespace1",
+    "non-matching-namespace",
+    "matching-variant1",
+    "matching-variant2",
+    "non-matching-variant",
+])
+def test_match(package_data, variant_index, requirement, expected):
+    """Return whether requirement is compatible with package."""
+    package = wiz.package.Package(
+        wiz.definition.Definition(package_data), variant_index=variant_index
+    )
+    assert wiz.utility.match(requirement, package) == expected
+
+
+@pytest.mark.parametrize("requirement, namespace, identifier", [
+    (Requirement("A"), None, "A"),
+    (Requirement("::A"), None, "A"),
+    (Requirement("foo::A"), "foo", "A"),
+    (Requirement("foo::bar::A"), "foo::bar", "A"),
+], ids=[
+    "without-namespace",
+    "qualified-without-namespace",
+    "with-namespace",
+    "with-long-namespace",
+])
+def test_extract_namespace(requirement, namespace, identifier):
+    """Extract namespace and identifier from requirement."""
+    assert wiz.utility.extract_namespace(requirement) == (namespace, identifier)
+
+
+@pytest.mark.parametrize("requirements1, requirements2, expected", [
+    ([], [], False),
+    (["bim > 3", "bah"], ["zim"], False),
+    (["bim > 3", "bah"], ["bim"], False),
+    (["bim > 3", "bah"], ["bim < 3"], True),
+], ids=[
+    "without-requirements",
+    "different-requirements",
+    "compatible-requirements",
+    "incompatible-requirements",
+])
+def test_check_conflicting_requirements(requirements1, requirements2, expected):
+    """Check whether some requirements are conflicting between packages."""
+    data1 = {"identifier": "foo"}
+    if len(requirements1):
+        data1["requirements"] = requirements1
+
+    data2 = {"identifier": "bar"}
+    if len(requirements2):
+        data2["requirements"] = requirements2
+
+    package1 = wiz.package.Package(wiz.definition.Definition(data1))
+    package2 = wiz.package.Package(wiz.definition.Definition(data2))
+
+    result = wiz.utility.check_conflicting_requirements(package1, package2)
+    assert result == expected
